<?php
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\CarouselManager\Api\Data\SlideInterfaceFactory;
use Pinpoint\CarouselManager\Api\Data\SlideSearchResultsInterfaceFactory;
use Pinpoint\CarouselManager\Api\SlideRepositoryInterface;
use Pinpoint\CarouselManager\Model\ResourceModel\Slide as ResourceSlide;
use Pinpoint\CarouselManager\Model\ResourceModel\Slide\CollectionFactory as SlideCollectionFactory;

class SlideRepository implements SlideRepositoryInterface
{

    protected $resource;

    protected $slideFactory;

    protected $slideCollectionFactory;

    protected $searchResultsFactory;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $dataSlideFactory;

    protected $extensionAttributesJoinProcessor;

    private $storeManager;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;

    /**
     * @param ResourceSlide $resource
     * @param SlideFactory $slideFactory
     * @param SlideInterfaceFactory $dataSlideFactory
     * @param SlideCollectionFactory $slideCollectionFactory
     * @param SlideSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceSlide $resource,
        SlideFactory $slideFactory,
        SlideInterfaceFactory $dataSlideFactory,
        SlideCollectionFactory $slideCollectionFactory,
        SlideSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->slideFactory = $slideFactory;
        $this->slideCollectionFactory = $slideCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataSlideFactory = $dataSlideFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Pinpoint\CarouselManager\Api\Data\SlideInterface $slide
    ) {
        $slideData = $this->extensibleDataObjectConverter->toNestedArray(
            $slide,
            [],
            \Pinpoint\CarouselManager\Api\Data\SlideInterface::class
        );
        
        $slideModel = $this->slideFactory->create()->setData($slideData);
        
        try {
            $this->resource->save($slideModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the slide: %1',
                $exception->getMessage()
            ));
        }
        return $slideModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($slideId)
    {
        $slide = $this->slideFactory->create();
        $this->resource->load($slide, $slideId);
        if (!$slide->getId()) {
            throw new NoSuchEntityException(__('Slide with id "%1" does not exist.', $slideId));
        }
        return $slide->getDataModel();
    }

    public function getByCarouselId($carouselId)
    {
        return $this->slideCollectionFactory->create()->addFieldToFilter('carousel_id', $carouselId)
            ->setOrder('sort_order', 'desc');
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->slideCollectionFactory->create();
        
        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Pinpoint\CarouselManager\Api\Data\SlideInterface::class
        );
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Pinpoint\CarouselManager\Api\Data\SlideInterface $slide
    ) {
        try {
            $slideModel = $this->slideFactory->create();
            $this->resource->load($slideModel, $slide->getSlideId());
            $this->resource->delete($slideModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Slide: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($slideId)
    {
        return $this->delete($this->get($slideId));
    }
}